@@ -6,7 +6,7 @@
 # DESCRIPTION: Provides an object-oriented interface to BibTeX entries.
 # CREATED    : March 1997, Greg Ward
 # MODIFIED   : 
-# VERSION    : $Id$
+# VERSION    : $Id: Entry.pm 8423 2010-02-13 20:40:27Z ambs $
 # COPYRIGHT  : Copyright (c) 1997-2000 by Gregory P. Ward.  All rights
 #              reserved.
 # 
@@ -19,33 +19,31 @@ package Text::BibTeX::Entry;
 require 5.004;                          # for isa, and delete on a slice
 
 use strict;
-use vars qw'$VERSION';
+use UNIVERSAL 'isa';
 use Carp;
 use Text::BibTeX qw(:metatypes :nodetypes);
 
-$VERSION = 0.76_02;
-
 =head1 NAME
 
 Text::BibTeX::Entry - read and parse BibTeX files
 
 =head1 SYNOPSIS
 
-   use Text::BibTeX::Entry;
+   use Text::BibTeX;            # do not use Text::BibTeX::Entry alone!
 
    # ...assuming that $bibfile and $newbib are both objects of class
    # Text::BibTeX::File, opened for reading and writing (respectively):
 
    # Entry creation/parsing methods:
-   $entry = Text::BibTeX::Entry->new();
+   $entry = new Text::BibTeX::Entry;
    $entry->read ($bibfile);
    $entry->parse ($filename, $filehandle);
    $entry->parse_s ($entry_text);
 
    # or:
-   $entry = Text::BibTeX::Entry->new( $bibfile );
-   $entry = Text::BibTeX::Entry->new( $filename, $filehandle );
-   $entry = Text::BibTeX::Entry->new( $entry_text );
+   $entry = new Text::BibTeX::Entry $bibfile;
+   $entry = new Text::BibTeX::Entry $filename, $filehandle;
+   $entry = new Text::BibTeX::Entry $entry_text;
    
    # Entry query methods
    warn "error in input" unless $entry->parse_ok;
@@ -86,7 +84,6 @@ Text::BibTeX::Entry - read and parse BibTeX files
    $entry->warn ($entry_warning);
    # or:
    $entry->warn ($field_warning, $field);
-   $entry->clone;
 
 =head1 DESCRIPTION
 
@@ -127,7 +124,7 @@ anything extra.
 
 =over 4
 
-=item new ([OPTS ,] [SOURCE])
+=item new ([SOURCE])
 
 Creates a new C<Text::BibTeX::Entry> object.  If the SOURCE parameter is
 supplied, it must be one of the following: a C<Text::BibTeX::File> (or
@@ -155,42 +152,20 @@ C<$filename> (error handling ignored):
 
    # good ol' fashioned filehandle and GLOB ref
    open (BIBFILE, $filename);
-   $entry = Text::BibTeX::Entry->new($filename, \*BIBFILE);
+   $entry = new Text::BibTeX::Entry ($filename, \*BIBFILE);
 
    # newfangled IO::File thingy
-   $file = IO::File->new($filename);
-   $entry = Text::BibTeX::Entry->new($filename, $file);
+   $file = new IO::File $filename;
+   $entry = new Text::BibTeX::Entry ($filename, $file);
 
 But using a C<Text::BibTeX::File> object is simpler and preferred:
 
-   $file  = Text::BibTeX::File->new($filename);
-   $entry = Text::BibTeX::Entry->new($file);
+   $file = new Text::BibTeX::File $filename;
+   $entry = new Text::BibTeX::Entry $file;
 
 Returns the new object, unless SOURCE is supplied and reading/parsing
 the entry fails (e.g., due to end of file) -- then it returns false.
 
-You may supply a reference to an option hash as first argument.
-Supported options are:
-
-=over 4 
-
-=item BINMODE
-
-Set the way Text::BibTeX deals with strings. By default it manages
-strings as bytes. You can set BINMODE to 'utf-8' to get NFC normalized
-
-Text::BibTeX::Entry->new(
-      { binmode => 'utf-8', normalization => 'NFD' },
-      $file });
-
-
-=item NORMALIZATION
-
-UTF-8 strings and you can customise the normalization with the NORMALIZATION option.
-
-=back
-
-
 =cut
 
 sub new
@@ -198,7 +173,6 @@ sub new
    my ($class, @source) = @_;
 
    $class = ref ($class) || $class;
-   
    my $self = {'file'     => undef,
                'type'     => undef,
                'key'      => undef,
@@ -206,28 +180,21 @@ sub new
                'metatype' => undef,
                'fields'   => [],
                'values'   => {}};
+
    bless $self, $class;
-
-   my $opts = {};
-   $opts = shift @source if scalar(@source) and ref $source[0] eq "HASH";
-   $opts->{ lc $_ } = $opts->{$_} for ( keys %$opts );
-   $self->{binmode} = 'utf-8'
-          if exists $opts->{binmode} && $opts->{binmode} =~ /utf-?8/i;
-   $self->{normalization} = $opts->{normalization} if exists $opts->{normalization};
-
    if (@source)
    {
       my $status;
 
-      if (@source == 1 && ref($source[0]) && $source[0]->isa ('Text::BibTeX::File'))
-      {
+      if (@source == 1 && isa ($source[0], 'Text::BibTeX::File'))
+      { 
          my $file = $source[0];
          $status = $self->read ($file);
          if (my $structure = $file->structure)
          {
             $self->{structure} = $structure;
             bless $self, $structure->entry_class;
-         }
+         }            
       }
       elsif (@source == 2 && defined $source[0] && ! ref $source[0] && fileno ($source[1]))
           { $status = $self->parse ($source[0], $source[1]) }
@@ -243,44 +210,6 @@ sub new
    $self;
 }
 
-=item clone
-
-Clone a Text::BibTeX::Entry object, returning the clone. This re-uses the reference to any
-Text::BibTeX::Structure or Text::BibTeX::File but copies everything else,
-so that the clone can be modified apart from the original.
-
-=cut
-
-sub clone
-{
-  my $self = shift;
-  my $clone = {};
-  # Use the same structure object - won't be changed
-  if ($self->{structure}) {
-    $clone->{structure} = $self->{structure};
-  }
-  # Use the same file object - won't be changed
-  if ($self->{file}) {
-    $clone->{file} = $self->{file}
-  }
-  # These might be changed so make copies
-  $clone->{binmode} = $self->{binmode};
-  $clone->{normalization} = $self->{normalization};
-  $clone->{type}     = $self->{type};
-  $clone->{key}      = $self->{key};
-  $clone->{status}   = $self->{status};
-  $clone->{metatype} = $self->{metatype};
-  $clone->{fields}   = [ map {$_} @{$self->{fields}} ];
-  while (my ($k, $v) = each %{$self->{values}}) {
-    $clone->{values}{$k} = $v;
-  }
-  while (my ($k, $v) = each %{$self->{lines}}) {
-    $clone->{lines}{$k} = $v;
-  }
-  bless $clone, ref($self);
-  return $clone;
-}
-
 =item read (BIBFILE)
 
 Reads and parses an entry from BIBFILE, which must be a
@@ -298,15 +227,11 @@ sub read
    my ($self, $source, $preserve) = @_;
    croak "`source' argument must be ref to open Text::BibTeX::File " .
          "(or descendant) object"
-      unless ($source->isa('Text::BibTeX::File'));
+      unless (isa ($source, 'Text::BibTeX::File'));
 
    my $fn = $source->{'filename'};
    my $fh = $source->{'handle'};
    $self->{'file'} = $source;        # store File object for later use
-   ## Propagate flags
-   for my $f (qw.binmode normalization.) {
-      $self->{$f} = $source->{$f} unless exists $self->{$f};
-   }
    return $self->parse ($fn, $fh, $preserve);
 }
 
@@ -363,7 +288,7 @@ sub _preserve
    $preserve = $self->{'file'}->preserve_values
       if ! defined $preserve && 
          defined $self->{'file'} &&
-           $self->{'file'}->isa ('Text::BibTeX::File');
+         isa ($self->{'file'}, 'Text::BibTeX::File');
    require Text::BibTeX::Value if $preserve;
    $preserve;
 }
@@ -428,39 +353,27 @@ context.)
 
 =item fieldlist ()
 
-Returns the list of fields in the entry.  
-
-B<WARNING> In scalar context, it no longer returns a
-reference to the object's own list of fields.
+Returns the list of fields in the entry.  In a scalar context, returns a
+reference to the object's own list of fields.  That way, you can change or
+reorder the field list with minimal interference from the class.  I'm not
+entirely sure if this is a good idea, so don't rely on it existing in the
+future; feel free to play around with it and let me know if you get bitten
+in dangerous ways or find this enormously useful.
 
 =cut
 
 sub parse_ok   { shift->{'status'}; }
 
-sub metatype   {
-    my $self = shift;
-    Text::BibTeX->_process_result( $self->{'metatype'}, $self->{binmode}, $self->{normalization} );
-}
+sub metatype   { shift->{'metatype'}; }
 
-sub type {
-    my $self = shift;
-    Text::BibTeX->_process_result( $self->{'type'}, $self->{binmode}, $self->{normalization} );
-}
+sub type       { shift->{'type'}; }
 
-sub key        { 
-  my $self = shift;
-  exists $self->{key}
-    ? Text::BibTeX->_process_result($self->{key}, $self->{binmode}, $self->{normalization})
-    : undef;
-}
+sub key        { shift->{'key'}; }
 
 sub num_fields { scalar @{shift->{'fields'}}; }
 
-sub fieldlist  { 
-  my $self = shift;
-  return map { Text::BibTeX->_process_result($_, $self->{binmode}, $self->{normalization})} @{$self->{'fields'}};
-}
-  
+sub fieldlist  { wantarray ? @{shift->{'fields'}} : shift->{'fields'}; }
+
 =item exists (FIELD)
 
 Returns true if a field named FIELD is present in the entry, false
@@ -532,24 +445,17 @@ sub exists
 {
    my ($self, $field) = @_;
 
-   exists $self->{values}{Text::BibTeX->_process_argument($field, $self->{binmode}, $self->{normalization})};
+   exists $self->{'values'}{$field};
 }
 
 sub get
 {
    my ($self, @fields) = @_;
 
-   my @x = @{$self->{'values'}}{map {Text::BibTeX->_process_argument($_, $self->{binmode}, $self->{normalization})} @fields};
-
-   @x = map {defined($_) ? Text::BibTeX->_process_result($_, $self->{binmode}, $self->{normalization}): undef} @x;
-
-   return (@x > 1) ? @x : $x[0];
+   @{$self->{'values'}}{@fields};
 }
 
-sub value { 
-  my $self = shift;
-  Text::BibTeX->_process_result($self->{value}, $self->{binmode}, $self->{normalization});
-}
+sub value { shift->{'value'} }
 
 
 =head2 Author name methods
@@ -645,19 +551,17 @@ sub split
 {
    my ($self, $field, $delim, $desc) = @_;
 
-   return unless $self->exists($field);
+   return unless $self->exists ($field);
    $delim ||= 'and';
    $desc ||= 'name';
 
+   my $filename = ($self->{'file'} && $self->{'file'}{'filename'});
+   my $line = $self->{'lines'}{$field};
+
 #   local $^W = 0                        # suppress spurious warning from 
 #      unless defined $filename;         # undefined $filename
-   Text::BibTeX::split_list($self->{values}{$field},
-                            $delim,
-                            ($self->{file} && $self->{file}{filename}),
-                            $self->{lines}{$field},
-                            $desc,
-                            {binmode       => $self->{binmode},
-                             normalization => $self->{normalization}});
+   Text::BibTeX::split_list ($self->{'values'}{$field}, $delim,
+                             $filename, $line, $desc);
 }
 
 sub names
@@ -675,8 +579,7 @@ sub names
 #      unless defined $filename;         # undefined $filename
    for $i (0 .. $#names)
    {
-      $names[$i] = Text::BibTeX::Name->new(
-        {binmode => $self->{binmode}, normalization => $self->{normalization}},$names[$i], $filename, $line, $i);
+      $names[$i] = new Text::BibTeX::Name ($names[$i], $filename, $line, $i);
    }
    @names;
 }
@@ -754,7 +657,7 @@ sub set_key
 {
    my ($self, $key) = @_;
 
-   $self->{'key'} = Text::BibTeX->_process_argument($key, $self->{binmode}, $self->{normalization});
+   $self->{'key'} = $key;
 }
 
 sub set
@@ -766,7 +669,7 @@ sub set
 
    while (@_)
    {
-      ($field,$value) = (shift,Text::BibTeX->_process_argument(shift, $self->{binmode}, $self->{normalization}));
+      ($field,$value) = (shift,shift);
       push (@{$self->{'fields'}}, $field)
          unless exists $self->{'values'}{$field};
       $self->{'values'}{$field} = $value;
@@ -871,14 +774,14 @@ sub print_s
       else                              # a Text::BibTeX::Value object
       {
          confess "value is a reference, but not to Text::BibTeX::Value object"
-            unless $value->isa ('Text::BibTeX::Value');
+            unless isa ($value, 'Text::BibTeX::Value');
          my @values = $value->values;
          foreach (@values)
          {
             $_ = $_->type == &BTAST_STRING ? '{' . $_->text . '}' : $_->text;
          }
-         return join (' # ', @values);
-     }
+         return join (' # ', @values);            
+      }
    }
 
    carp "entry type undefined" unless defined $self->{'type'};
@@ -891,8 +794,8 @@ sub print_s
    {
       carp "entry key undefined" unless defined $self->{'key'};
       $output = sprintf ("@%s{%s,\n",
-                         $self->{'type'} || '',
-                         $self->{'key'}  || '');
+                         $self->{'type'} || '', 
+                         $self->{'key'} || '');
    }
    elsif ($self->{'metatype'} == &BTE_MACRODEF)
    {
@@ -923,13 +826,13 @@ sub print_s
       $output .= "  $field = ";
       $output .= value_to_string ($value);
 
-      $output .= ",\n";
+      $output .= ',' if @fields;        # more fields yet to come
+      $output .= "\n";
    }
 
    # Tack on the last line, and we're done!
    $output .= "}\n\n";
-   
-   Text::BibTeX->_process_result($output, $self->{binmode}, $self->{normalization});
+   $output;
 }
 
 =back
